<?php

class Akismet
{
	/**
	 * The API key being verified for use with the API
	 * This variable is set from the config file (/system/application/config/akismet.php)
	 * @access private
	 * @var string
	 */
	private $key;
	
	/**
	 * The front page or home URL of the instance making the request. For a blog or wiki this 
	 * would be the front page. Note: Must be a full URI, including http://.
	 * This variable is set from the config file (/system/application/config/akismet.php)
	 * @access private
	 * @var string
	 */
	private $blog;
	
	/**
	 * User agent information of this application, not the comment poster.
	 * This variable is set from the config file (/system/application/config/akismet.php)
	 * @access public
	 * @var string
	 */
	private $app_user_agent;
	
	/**
	 * IP address of the comment submitter.
	 * @access public
	 * @var string
	 */
	public $user_ip;
	
	/**
	 * User agent information.
	 * @access public
	 * @var string
	 */
	public $user_agent;
	
	/**
	 * The content of the HTTP_REFERER header should be sent here.
	 * @access public
	 * @var string
	 */
	public $referrer;
	
	/**
	 * The permanent location of the entry the comment was submitted to.
	 * @access public
	 * @var string
	 */
	public $permalink;
	
	/**
	 * May be blank, comment, trackback, pingback, or a made up value like "registration".
	 * @access public
	 * @var string
	 */
	public $comment_type;
	
	/**
	 * Submitted name with the comment
	 * @access public
	 * @var string
	 */
	public $comment_author;
	
	/**
	 * Submitted email address
	 * @access public
	 * @var string
	 */
	public $comment_author_email;
	
	/**
	 * Commenter URL.
	 * @access public
	 * @var string
	 */
	public $comment_author_url;
	
	/**
	 * The content that was submitted.
	 * @access public
	 * @var string
	 */
	public $comment_content;
	
	/**
	 * CodeIgniter object
	 * @access private
	 * @var string
	 */
	private $CI;
	
	/**
	 * Constructor - set some variables and configuration settings
	 */
	public function __construct()
	{
		/* make the CI object available */
		$this->CI =& get_instance();
		
		/* load the config settings */
		$this->CI->config->load('akismet', TRUE);
		
		$this->app_user_agent = $this->CI->config->item('app_user_agent', 'akismet');
		$this->key = $this->CI->config->item('key', 'akismet');
		$this->blog = $this->CI->config->item('blog', 'akismet');
		
		/* Akismet development docs request we specify a user agent - 
			we only need to set this once */
		$this->snoopy->agent = $this->CI->config->item('app_user_agent', 'akismet');
	}
	
	/**
	 * Verify the API key
	 * @param mixed $config the array used to set the config values
	 */
	public function init($config)
	{
		/* create an array with the fields and dummy values */
		$fields = array('key' => 1, 'blog' => 1, 'user_ip' => 1, 'user_agent' => 1, 'referrer' => 1, 
		'permalink' => 1, 'comment_type' => 1, 'comment_author' => 1, 'comment_author_email' => 1, 
		'comment_author_url' => 1, 'comment_content' => 1, 'send_server_var' => 1);
		
		/* now intersect the fields in both arrays to make sure we are only setting certain values */
		$config = array_intersect_key($config, $fields);
		
		foreach ($config as $field => $value)
		{
			$this->field = $value;
		}
	}
	
	/**
	 * Verify the API key
	 * @param mixed $debug (byref) the debug information to be returned just in case
	 */
	public function verify_key(&$debug = '')
	{
		$post = 'key=' . $this->key . '&blog=' . $this->blog;
		
		return $this->http_post($post, '/1.1/verify-key', &$debug);
	}
	
	/**
	 * Checks to see if the comment is spam
	 * @param mixed $debug (byref) the debug information to be returned just in case
	 */
	public function is_spam(&$debug = '')
	{
		return $this->http_post($this->build_post(), '/1.1/comment-check', &$debug);
	}
	
	/**
	 * Tell Akismet that a comment is spam that was thought to be ham
	 * @param mixed $debug (byref) the debug information to be returned just in case
	 */
	public function submit_spam(&$debug = '')
	{
		return $this->http_post($this->build_post(), '/1.1/submit-spam', &$debug);
	}
	
	/**
	 * Tell Akismet that a comment is ham that was thought to be spam
	 * @param mixed $debug (byref) the debug information to be returned just in case
	 */
	public function submit_ham(&$debug = '')
	{
		return $this->http_post($this->build_post(), '/1.1/submit-ham', &$debug);
	}
	
	/**
	 * Build a $post array based on the object variables
	 * @return mixed
	 */
	private function build_post()
	{
		$post = array();
		if ($this->send_server_var) $post = $_SERVER;
		$post['key'] = $this->key;
		$post['blog'] = $this->blog;
		$post['user_ip'] = $this->user_ip;
		$post['user_agent'] = $this->user_agent;
		$post['referrer'] = $this->referrer;
		$post['permalink'] = $this->permalink;
		$post['comment_type'] = $this->comment_type;
		$post['comment_author'] = $this->comment_author;
		$post['comment_author_email'] = $this->comment_author_email;
		$post['comment_author_url'] = $this->comment_author_url;
		$post['comment_content'] = $this->comment_content;
		
		$post_request = array();
		foreach ($post as $field => $value)
		{
			$post_request[] = $field . '=' . $value;
		}
		$post_request = implode('&', $post_request);
		
		return $post_request;
	}
	
	/**
	 * send a request to Akismet
	 * @param mixed $data the var=value pairs separated by '&'
	 * @param string $path the path to send the data to
	 * @param mixed $debug (byref) the debug information to be returned just in case
	 * @return string
	 */
	private function http_post($data, $path, &$debug = '') 
	{
		$host = $this->key . '.rest.akismet.com';
		
		$http_request  = "POST $path HTTP/1.0\r\n";
		$http_request .= "Host: $host\r\n";
		$http_request .= "Content-Type: application/x-www-form-urlencoded; charset=utf-8\r\n";
		$http_request .= "Content-Length: " . strlen($data) . "\r\n";
		$http_request .= "User-Agent: " . $this->user_agent . "\r\n";
		$http_request .= "\r\n";
		$http_request .= $data;

		$response = '';
		if (false !== ($fs = fsockopen($host, 80, $errno, $errstr, 3)))
		{
			fwrite($fs, $http_request);
			while (!feof($fs))
			{
				$response .= fgets($fs, 1160); // One TCP-IP packet
			}
			fclose($fs);
			$response = explode("\r\n\r\n", $response, 2);
		}
		
		$debug = $response[0];
		return $response[1];
	}  
}

/* ?> */
